mod font;
mod panel;
mod ext;
mod title;
mod tray;
mod util;
mod widget;
mod x11;

use chan;
use chan_signal;
use config::Config;

use std::thread;

pub const SIZE: u16 = 20;

pub fn ui_main(signal: chan::Receiver<chan_signal::Signal>, cfg: &Config) -> i32 {
    if let Some(conn) = x11::Connection::new() {
        let (tx, rx) = chan::sync(0);
        {
            let conn = conn.clone_connection();
            thread::spawn(move || {
                loop {
                    match conn.wait_for_event() {
                        Some(event) => { tx.send(event); },
                        None => { break; }
                    }
                }
            });
        }

        let (mut panel, panel_rx) = panel::Panel::new(conn, cfg);

        let title = title::Title::new(panel.conn.clone(), panel.make_draw_context());
        panel.add_left_widget(Box::new(title));

        let tray = tray::Tray::new(panel.tx.clone(), panel.conn.clone(), panel.window);
        panel.add_right_widget(Box::new(tray));

        panel.create();

        loop {
            chan_select!(
                panel_rx.recv() => {
                    panel.relayout();
                },
                rx.recv() -> event => {
                    if panel.handle_event(event.unwrap()) {
                        println!("Exiting");
                        break;
                    }
                },
                signal.recv() => {
                    panel.finish();
                }
            );
        }
    }
    0
}
